/*
(c) 2006 Mauro Laurenti
All rights reserved
www.laurtec.com

 This library contains all the functions that are handy for the controlling 			
 of an LCD with a 44780 Hitachi controller							
 This header file has been tested with 20MHz Quartz. You have to change the Delay 		
 function whether you are using a different Quartz.						
 												

 Available Functions 

void   OpenLCD ()
void   ClearLCD ()
void   CursorLCD (char,char)		1=ONcursor 0=OFF cursor;  1=ON blinking 0=OFF Blinking
void   WriteCharLCD (char)
void   HomeLCD ()
void   Line2LCD ()
void   ShiftLCD (char)				0=left	1=right
void   ShiftCursorLCD (char);			0=left	1=right
void   WriteStringLCD (" Hello World ")
void   WriteVarLCD(char *buffer)


 to use these functions for your pourpose you must set up properly 
 the following LCD constants and the pins of the PORT/s that you will use.
 You can use any pin you want!

*/



//*********************************************************************************************
// LCD constants

#define LCD_D0 PORTBbits.RB4	// you must set this pin as output
#define LCD_D1 PORTBbits.RB5	// you must  set this pin as output
#define LCD_D2 PORTBbits.RB6	// you must  set this pin as output
#define LCD_D3 PORTBbits.RB7	// you must  set this pin as output
#define LCD_RS PORTBbits.RB2	// you must  set this pin as output
#define LCD_E  PORTBbits.RB3	// you must  set this pin as output

//*********************************************************************************************
//Other Constants

#define LEFT 0
#define RIGHT 1

//*********************************************************************************************
//this function accept in input the number of us for the dalay
//the number of nop inside the loop are trimmed for 20MHz Quartz

void delay (int attesa)
{	int i;
	for (i =0; i<attesa;i++)
		{
			_asm 
			nop
			nop
			nop
			nop
			_endasm
		}	
}

//*********************************************************************************************
//This function create a pulse on the Pin Enable of the LCD to allow the sending of the command 

void Epulse ()
{
	LCD_E = 1;
	delay (350);
	LCD_E = 0;
	delay (100);		
}

//*********************************************************************************************
//Send a command. The input are the 8 command bits splitted in two 4 bit command
//You have to send the 4 high bit and than the 4 low bit

void SendCommand (unsigned char D3, unsigned char D2, unsigned char D1, unsigned char D0)
{
	LCD_D0 = D0;
	LCD_D1 = D1;
	LCD_D2 = D2;
	LCD_D3 = D3;
	Epulse ();
	delay (20);	
}

//*********************************************************************************************

void Line2LCD()
{
	SendCommand (1,1,0,0);	// the cursor is moved to the second line
	SendCommand (0,0,0,0); 		
}

//*********************************************************************************************

void HomeLCD()
{
	SendCommand (0,0,0,0);	// the cursor is moved at the beginning of the LCD
	SendCommand (0,0,1,0); 		
}


//*********************************************************************************************

void ShiftLCD(char L_R)
{	
	SendCommand (0,0,0,1);	// the LCD is shifted on the left or on the right
	SendCommand (1,L_R,0,0); 		
}


//*********************************************************************************************

void ShiftCursorLCD(char L_R)
{
	SendCommand (0,0,0,1);	// the cursor is shifted on the left or on the right
	SendCommand (0,L_R,0,0); 		
}

//*********************************************************************************************
// write a character to the LCD

void WriteCharLCD (char Element)
{	
	unsigned char D3,D2,D1,D0;
	
		LCD_RS = 1;
					
		D3 = (Element & 0b10000000) >> 7;	// Splitting of the first nibble
		D2 = (Element & 0b01000000) >> 6;
		D1 = (Element & 0b00100000) >> 5;
		D0 = (Element & 0b00010000) >> 4;
		
		SendCommand (D3,D2,D1,D0);
		
		D3 = (Element & 0b00001000) >> 3;	// Splitting of the second nibble
		D2 = (Element & 0b00000100) >> 2;
		D1 = (Element & 0b00000010) >> 1;
		D0 = (Element & 0b00000001);
			
		SendCommand (D3,D2,D1,D0);
		
		LCD_RS = 0;
}

//*********************************************************************************************
// This function allow to write a costant string to the LCD

void WriteStringLCD(const rom char *buffer)
{
        while(*buffer)                  // Write data to LCD up to null
        {
                WriteCharLCD(*buffer);  // Write character to LCD
                buffer++;               // Increment buffer
        }
        return;
}

//*********************************************************************************************
// This function allow to write a string Variable to the LCD 

void WriteVarLCD(char *buffer)
{
        while(*buffer)                  // Write data to LCD up to null
        {
                WriteCharLCD(*buffer); // Write character to LCD
                buffer++;               // Increment buffer
        }
        return;
}
//*********************************************************************************************
// this function clean the LCD 

void ClearLCD ()
{
	SendCommand (0,0,0,0);	
	SendCommand (0,0,0,1);		
}

//*********************************************************************************************
//This function turn OFF or ON the cursor of the display and its blinking
//0=Turn OFF	1= Turn ON

void CursorLCD(char ON_OFF,char Blink)
{	
	SendCommand (0,0,0,0);	
	SendCommand (1,1,ON_OFF,Blink);			
							
}



//*********************************************************************************************
//Reset the LCD

void  OpenLCD()
{	
	// you have to set the port of the LCD properly

	LCD_RS = 0x00;
	LCD_E = 0x00;

	delay (1000);
	SendCommand (0,0,1,1);
	delay (1000);		// delay is more than 4mS
	SendCommand (0,0,1,1);
	delay (1000);		// delay is more than 4mS
	SendCommand (0,0,1,1);
	delay (1000);		// delay is more than 4mS
	SendCommand (0,0,1,0);
	SendCommand (0,0,1,0);
	SendCommand (1,0,0,0);
	SendCommand (0,0,0,0);
	SendCommand (1,1,1,0);
	CursorLCD (0,0);
	ClearLCD ();	
}
