/*
(c) 2006 Mauro Laurenti
All rights reserved
www.laurtec.com

 This library contains all the functions that are handy for the controlling 			
 of an LCD with a 44780 Hitachi controller							
 This header file has been tested with 20MHz Quartz. You have to change the Delay 		
 function whether you are using a different Quartz.						
 												

 Available Functions 

void   OpenLCD ()
void   ClearLCD ()
void   CursorLCD (char,char)		1=ONcursor 0=OFF cursor;  1=ON blinking 0=OFF Blinking
void   WriteCharLCD (char)
void   HomeLCD ()
void   Line2LCD ()
void   ShiftLCD (char)				0=left	1=right
void   ShiftCursorLCD (char);			0=left	1=right
void   WriteStringLCD (" Hello World ")
void   WriteVarLCD(char *buffer)


*/



//*********************************************************************************************
//Constants

#define LEFT 0
#define RIGHT 1

//*********************************************************************************************
//this function accept in input the number of us for the dalay
//the number of nop inside the loop are trimmed for 20MHz Quartz

void delay (int attesa)
{	int i;
	for (i =0; i<attesa;i++)
		{				
			_asm 
			nop
			nop
			nop
			nop
			_endasm
		}	
}

//*********************************************************************************************
//This function create a pulse on the Pin Enable of the LCD to allow the sending of the command 

void Epulse ()
{
	PORTDbits.RD3 = 0x01;	
	delay (350);
	
	PORTDbits.RD3 = 0x00;	
	delay (100);		
}

//*********************************************************************************************
//Send a command. The input are the 8 command bits splitted in two 4 bit command
//You have to send the 4 high bit and than the 4 low bit

void SendCommand (int Dato)
{
	PORTD = PORTD & 0b00001111;
	PORTD = PORTD | Dato;
 	Epulse ();
	delay (20);	
}

//*********************************************************************************************

void Line2LCD()
{
	SendCommand (0b11000000);	// the cursor is moved to the second line
	SendCommand (0b00000000); 		
}

//*********************************************************************************************

void HomeLCD()
{
	SendCommand (0b00000000);	// the cursor is moved at the beginning of the LCD
	SendCommand (0b00100000); 		
}


//*********************************************************************************************

void ShiftLCD(char L_R)
{	
	SendCommand (0b00010000);	// the LCD is shifted on the left or on the right
	if (L_R)
		SendCommand (0b11000000);
		else
		SendCommand (0b10000000);
	
}


//*********************************************************************************************

void ShiftCursorLCD(char L_R)
{
	SendCommand (0b00010000);	// the cursor is shifted on the left or on the right
	if (L_R)
		SendCommand (0b01000000);
		else
		SendCommand (0b00000000); 		
}

//*********************************************************************************************
// write a character to the LCD

void WriteCharLCD (char Element)
{	
		char buffer; 	
				
		buffer = Element & 0b11110000;			
		
		PORTD = PORTD & 0b00001111;
		PORTD = PORTD | buffer | 0b00001100;
 		delay (350);
 		PORTD = PORTD & 0b11110111;
		delay (120);	
		
		buffer = Element & 0b00001111;		
		buffer = buffer << 4;
		
		PORTD = PORTD & 0b00001111 ;
		PORTD = PORTD | buffer | 0b00001100;
 		delay (350);
 		PORTD = PORTD & 0b11110111;
		delay (120);	
		
		_asm
		BCF PORTD,2,0
		_endasm
}

//*********************************************************************************************
// This function allow to write a costant string to the LCD

void WriteStringLCD(const rom char *buffer)
{
        while(*buffer)                  // Write data to LCD up to null
        {
                WriteCharLCD(*buffer);  // Write character to LCD
                buffer++;               // Increment buffer
        }
        return;
}

//*********************************************************************************************
// This function allow to write a string Variable to the LCD 

void WriteVarLCD(char *buffer)
{
        while(*buffer)                  // Write data to LCD up to null
        {
                WriteCharLCD(*buffer); // Write character to LCD
                buffer++;               // Increment buffer
        }
        return;
}
//*********************************************************************************************
// this function clean the LCD 

void ClearLCD ()
{
	SendCommand (0b00000000);	
	SendCommand (0b00010000);
}

//*********************************************************************************************
//This function turn OFF or ON the cursor of the display and its blinking
//0=Turn OFF	1= Turn ON

void CursorLCD(char ON_OFF,char Blink)
{	char buffer =0b11000000;
	
	SendCommand (0b00000000);
	
	if (ON_OFF)
		buffer = buffer | 0b00100000;
	if (Blink)
		buffer = buffer | 0b00010000;
	
	SendCommand (buffer);			
							
}
//*********************************************************************************************
//Reset the LCD in 4 bit format.
//The data inside the byte are the 4 more significant bits

void  OpenLCD()
{	
	// you have to set the port of the LCD properly

	_asm
		BCF PORTD,2,0
		BCF PORTD,3,0
	_endasm


	delay (1000);
	SendCommand (0b00110000);
	delay (1000);		// delay is more than 4mS
	SendCommand (0b00110000);
	delay (1000);		// delay is more than 4mS
	SendCommand (0b00110000);
	delay (1000);		// delay is more than 4mS
	SendCommand (0b00100000);
	SendCommand (0b00100000);
	SendCommand (0b10000000);
	SendCommand (0b00000000);
	SendCommand (0b11100000);
	CursorLCD (0,0);
	ClearLCD ();	
}
