
/*

(c) 2006 Mauro Laurenti
All rights reserved
www.laurtec.com

 Collection of available functions

 unsigned char WriteSeconds (unsigned char Seconds)
 unsigned char ReadSeconds (void)
 unsigned char WriteMinutes (unsigned char Minutes)
 unsigned char ReadMinutes (void)
 unsigned char WriteHours (unsigned char Hours)
 unsigned char ReadHours (void)
 
 unsigned char* ReadTimeSeconds (void)
 unsigned char* ReadTime (void)
 
 unsigned char WriteDays (unsigned char Days)
 unsigned char ReadDays (void)
 unsigned char WriteWeekDays (unsigned char WeekDays)
 unsigned char ReadWeekDays (void)
 unsigned char WriteMonths (unsigned char Months)
 unsigned char ReadMonths (void)
 unsigned char WriteYears (unsigned char Years)
 unsigned char ReadYears (void)
 
 unsigned char* ReadDate (void)
 
 unsigned char WriteMinutesAlarm (unsigned char Minutes,unsigned char AlarmEnable)
 unsigned char WriteHoursAlarm (unsigned char Hours,unsigned char AlarmEnable)
 unsigned char WriteDaysAlarm (unsigned char Days,unsigned char AlarmEnable)
 unsigned char WriteWeekDaysAlarm (unsigned char WeekDays,unsigned char AlarmEnable)
 
 unsigned char EnableInt (void)
 unsigned char DisableAllInt (void)
 unsigned char IsAllarmON (void) 

*/

//Constants.

#define	DO 0x00	//English SU
#define	LU 0x01 //English MO
#define	MA 0x02 //English TU
#define	ME 0x03 //English WE
#define	GI 0x04 //English TR
#define	VE 0x05 //English FR
#define	SA 0x06 //English SA


// you have to change these addresses whether you change the device
#define WRITE_ADD 0xA2			
#define READ_ADD 0xA3


#include <i2c.h>


//**********************************************************************************************

#define Enable_ON  0b00000000	//enable value for the allarm bit (day,month,dayweek)
#define Enable_OFF 0b10000000

//**********************************************************************************************
//Seconds Functions
 
unsigned char WriteSeconds (unsigned char Seconds)
{	
	return (EEByteWrite (WRITE_ADD,0x02,Seconds));	//0x03 is the Seconds address
	
}

unsigned char ReadSeconds (void)
{	unsigned char Seconds;
	Seconds = EERandomRead (WRITE_ADD,0x02);  
	Seconds = Seconds & 0b01111111;	// I put to 0 the not significant bits
	return (Seconds);
}


//**********************************************************************************************
//Minutes Functions

unsigned char WriteMinutes (unsigned char Minutes)
{	
	return (EEByteWrite (WRITE_ADD,0x03,Minutes));	//0x03 is the minutes address
	
}

unsigned char ReadMinutes (void)
{	unsigned char Minutes;
	Minutes = EERandomRead (WRITE_ADD,0x03);  
	Minutes = Minutes & 0b01111111;	// I put to 0 the not significant bits
	return (Minutes);
}

//**********************************************************************************************
//Hours Functions

unsigned char WriteHours (unsigned char Hours)
{	
	return (EEByteWrite (WRITE_ADD,0x04,Hours));	//0x04 is the Hours address
	
}

unsigned char ReadHours (void)
{	unsigned char Hours;
	Hours = EERandomRead (WRITE_ADD,0x04);  
	Hours = Hours & 0b00111111;	// I put to 0 the not significant bits
	return (Hours);
}

//**********************************************************************************************
//Read Time and return string with the time and seconds HH:MM.ss

unsigned char* ReadTimeSeconds (void)
{	static unsigned char Time[9];
	unsigned char value;

	value = ReadHours();	
	Time[1] = (value & 0b00001111)+48; // with +48 I convert the number in ASCII number
	Time[0] = (value >> 4)+48;
	
	Time[2] = ':';
	
	value = ReadMinutes();	
	Time[4] = (value & 0b00001111)+48;
	Time[3] = (value >> 4)+48;
	
	Time[5] = '.';

	value = ReadSeconds();	
	Time[7] = (value & 0b00001111)+48;
	Time[6] = (value >> 4)+48;
	
	Time[8] = '\0';
		
	return (Time);
	
}


//**********************************************************************************************
//Read Time return a string with time without the seconds HH:MM

unsigned char* ReadTime (void)
{	static unsigned char Time[6];
	unsigned char value;

	value = ReadHours();	
	Time[1] = (value & 0b00001111)+48; // with +48 I convert the number in ASCII number
	Time[0] = (value >> 4)+48;
	
	Time[2] = ':';
	
	value = ReadMinutes();	
	Time[4] = (value & 0b00001111)+48;
	Time[3] = (value >> 4)+48;
	
	
	Time[5] = '\0';
		
	return (Time);
	
}

//**********************************************************************************************
//Days Functions

unsigned char WriteDays (unsigned char Days)
{	
	return (EEByteWrite (WRITE_ADD,0x05,Days));	//0x05 is the Days address
	
}

unsigned char ReadDays (void)
{	unsigned char Days;
	Days = EERandomRead (WRITE_ADD,0x05);  
	Days = Days & 0b00111111;	// I put to 0 the not significant bits
	return (Days);
}

//**********************************************************************************************
//WeekDays Functions

unsigned char WriteWeekDays (unsigned char WeekDays)
{	
	return (EEByteWrite (WRITE_ADD,0x06,WeekDays));	//0x06 is the WeekDays address
	
}

unsigned char ReadWeekDays (void)
{	unsigned char WeekDays;
	WeekDays = EERandomRead (WRITE_ADD,0x06);  
	WeekDays = WeekDays & 0b00000111;	// I put to 0 the not significant bits
	return (WeekDays);
}

//**********************************************************************************************
//Months Functions

unsigned char WriteMonths (unsigned char Months)
{	
	return (EEByteWrite (WRITE_ADD,0x07,Months));	//0x07 is the Months address
	
}

unsigned char ReadMonths (void)
{	unsigned char Months;
	Months = EERandomRead (WRITE_ADD,0x07);  
	Months = Months & 0b00011111;	// I put to 0 the not significant bits
	return (Months);
}

//**********************************************************************************************
//Year Functions

unsigned char WriteYears (unsigned char Years)
{	
	return (EEByteWrite (WRITE_ADD,0x08,Years));	//0x08 is the Years address
	
}

unsigned char ReadYears (void)
{	unsigned char Years;
	Years = EERandomRead (WRITE_ADD,0x08);  
	return (Years);
}

//**********************************************************************************************
//Read Date in Italian and return a string in this Format DD/MM/YY

unsigned char* ReadDate (void)
{	static unsigned char Date[9];
	unsigned char value;

	value = ReadDays();	
	Date[1] = (value & 0b00001111)+48; // with +48 I convert the number in ASCII number
	Date[0] = (value >> 4)+48;
	
	Date[2] = '/';
	
	value = ReadMonths();	
	Date[4] = (value & 0b00001111)+48;
	Date[3] = (value >> 4)+48;
	
	Date[5] = '/';


	value = ReadYears();	
	Date[7] = (value & 0b00001111)+48;
	Date[6] = (value >> 4)+48;
	
	Date[8] = '\0';
		
	return (Date);
	
}


//**********************************************************************************************
//MinutesAlarm

unsigned char WriteMinutesAlarm (unsigned char Minutes, unsigned char AlarmEnable)
{	
	Minutes = Minutes + AlarmEnable; //I activate AE if required
	return (EEByteWrite (WRITE_ADD,0x09,Minutes));	//0x09 is the minutes alarm address
	
}

//**********************************************************************************************
//HourAlarm

unsigned char WriteHoursAlarm (unsigned char Hours, unsigned char AlarmEnable)
{	
	Hours = Hours + AlarmEnable; //I activate AE if required
	return (EEByteWrite (WRITE_ADD,0x0A,Hours));	//0x0A is the minutes alarm address
	
}

//**********************************************************************************************
//DayAllarm

unsigned char WriteDaysAlarm (unsigned char Days, unsigned char AlarmEnable)
{	
	Days = Days + AlarmEnable; //I activate AE if required
	return (EEByteWrite (WRITE_ADD,0x0B,Days));	//0x0B is the minutes alarm address
	
}

//**********************************************************************************************
//WeekDaysAlarm

unsigned char WriteWeekDaysAlarm (unsigned char WeekDays, unsigned char AlarmEnable)
{	
	WeekDays = WeekDays + AlarmEnable; //I activate AE if required
	return (EEByteWrite (WRITE_ADD,0x0C,WeekDays));	//0x0C is the minutes alarm address
	
}


//**********************************************************************************************
//Enable Allarm Intterrupt
//this function set to 0 the others bit!

unsigned char EnableAlarmInt (void)
{	
	
	return (EEByteWrite (WRITE_ADD,0x01,0x02));	
	
}


//**********************************************************************************************
//Disable All Alarm Intterrupts

unsigned char DisableAllInt (void)
{	
	
	return (EEByteWrite (WRITE_ADD,0x01,0x00));	
	
}
//**********************************************************************************************
//Control the interrupt bit
//Whether the alarm is on the AF flag is automatically cleaned

unsigned char IsAlarmON (void)
{	
	unsigned char value;
	value = EERandomRead (WRITE_ADD,0x01);  
	if (value & 0x08)	//just AF bit is controlled
		{
			value = value & 0xF7;	
			EEByteWrite (WRITE_ADD,0x01,value); // I clean AF bit without cange the register  
			return (1);	
		}
		else
		{
			return (0);	
		}
		
}

