
/****************************************************************************
(c) 2006 Mauro Laurenti
All rights reserved
www.laurtec.com

* These functions allow the user to read/write inside the eeprom such as the 24LC512.
* The control variable is the address of the eeprom with the bit R/'W set to 0 

*/


#include <i2c.h>


unsigned char WriteEEprom( unsigned char control, unsigned char addressH,unsigned char addressL, unsigned char data )
{
  IdleI2C();                      // ensure module is idle
  StartI2C();                     // initiate START condition
  while ( SSPCON2bits.SEN );      // wait until start condition is over 
  if ( PIR2bits.BCLIF )           // test for bus collision
  {
    return ( -1 );                // return with Bus Collision error 
  }
  else                            // start condition successful
  {
    if ( WriteI2C( control ) )    // write byte - R/W bit should be 0
    {
      return ( -3 );              // set error for write collision
    }

    IdleI2C();                    // ensure module is idle
    if ( !SSPCON2bits.ACKSTAT )   // test for ACK condition received
    { 
      if ( WriteI2C( addressH ) )  // write word address for EEPROM
      {
        return ( -3 );            // set error for write collision
      }

      IdleI2C();                  // ensure module is idle
      if ( !SSPCON2bits.ACKSTAT ) // test for ACK condition received
      { 
        if ( WriteI2C( addressL ) )   // data byte for EEPROM
        {
          return ( -3 );          // set error for write collision
        }
      }
      
      IdleI2C();                  // ensure module is idle
      if ( !SSPCON2bits.ACKSTAT ) // test for ACK condition received
      { 
        if ( WriteI2C( data ) )   // data byte for EEPROM
        {
          return ( -3 );          // set error for write collision
        }
      }
      
      else
      {
        return ( -2 );            // return with Not Ack error condition   
      }
    }
    else
    {
      return ( -2 );              // return with Not Ack error condition   
    }
  }

  IdleI2C();                      // ensure module is idle  
  StopI2C();                      // send STOP condition
  while ( SSPCON2bits.PEN );      // wait until stop condition is over 
  if ( PIR2bits.BCLIF )           // test for bus collision
  {
    return ( -1 );                // return with Bus Collision error 
  }
  return ( 0 );                   // return with no error
}


/****************************************************************************
* This function allow the user to read inside the eeprom such as the 24LC512.
* The control variable is the address of the eeprom with the bit R/'W set to 0 
*/

unsigned char ReadEEprom( unsigned char control, unsigned char addressH,unsigned char addressL )
{
  IdleI2C();                      // ensure module is idle
  StartI2C();                     // initiate START condition
  while ( SSPCON2bits.SEN );      // wait until start condition is over 
  if ( PIR2bits.BCLIF )           // test for bus collision
  {
    return ( -1 );                // return with Bus Collision error 
  }
  else
  {
    if ( WriteI2C( control ) )    // write 1 byte
    {
      return ( -3 );              // return with write collision error
    }

    IdleI2C();                    // ensure module is idle
    if ( !SSPCON2bits.ACKSTAT )   // test for ACK condition, if received
    {
      if ( WriteI2C( addressH ) )  // WRITE word address for EEPROM
      {
        return ( -3 );            // return with write collision error
      }
    } 


    IdleI2C();                    // ensure module is idle
    if ( !SSPCON2bits.ACKSTAT )   // test for ACK condition, if received
    {
      if ( WriteI2C( addressL ) )  // WRITE word address for EEPROM
      {
        return ( -3 );            // return with write collision error
      }
	}

      IdleI2C();                  // ensure module is idle
      if ( !SSPCON2bits.ACKSTAT ) // test for ACK condition, if received
      {
        RestartI2C();             // generate I2C bus restart condition
        while ( SSPCON2bits.RSEN );// wait until re-start condition is over 
        if ( PIR2bits.BCLIF )     // test for bus collision
        {
          return ( -1 );          // return with Bus Collision error 
        }
                          
        if ( WriteI2C( control+1 ) )// write 1 byte - R/W bit should be 1
        {
          return ( -3 );          // return with write collision error
        }

        IdleI2C();                // ensure module is idle
        if ( !SSPCON2bits.ACKSTAT )// test for ACK condition, if received
        {
          SSPCON2bits.RCEN = 1;       // enable master for 1 byte reception
          while ( SSPCON2bits.RCEN ); // check that receive sequence is over
          NotAckI2C();              // send ACK condition
          while ( SSPCON2bits.ACKEN ); // wait until ACK sequence is over 
          StopI2C();              // send STOP condition
          while ( SSPCON2bits.PEN ); // wait until stop condition is over 
          if ( PIR2bits.BCLIF )   // test for bus collision
          {
           return ( -1 );         // return with Bus Collision error 
          }
        }
        else
        {
          return ( -2 );          // return with Not Ack error
        }

      }
      else
      {
        return ( -2 );            // return with Not Ack error
      }
    }

  
  return ( SSPBUF );     // return with data
}
